@@ -42,13 +42,13 @@ import orca.orca_state as orca_state
 import orca.speech as speech
 import orca.settings as settings
 import orca.settings_manager as settings_manager
+import orca.structural_navigation as structural_navigation
 
 from .braille_generator import BrailleGenerator
 from .formatting import Formatting
 from .script_utilities import Utilities
 from .spellcheck import SpellCheck
 from .speech_generator import SpeechGenerator
-from .structural_navigation import StructuralNavigation
 
 _settingsManager = settings_manager.getManager()
 
@@ -106,14 +106,14 @@ class Script(default.Script):
         """Returns the 'structural navigation' class for this script.
         """
         types = self.getEnabledStructuralNavigationTypes()
-        return StructuralNavigation(self, types, enabled=False)
+        return structural_navigation.StructuralNavigation(self, types, enabled=False)
 
     def getEnabledStructuralNavigationTypes(self):
         """Returns a list of the structural navigation object types
         enabled in this script.
         """
 
-        enabledTypes = [StructuralNavigation.TABLE_CELL]
+        enabledTypes = [structural_navigation.StructuralNavigation.TABLE_CELL]
 
         return enabledTypes
 
@@ -290,27 +290,6 @@ class Script(default.Script):
         prefs.update(self.spellcheck.getPreferencesFromGUI())
         return prefs
 
-    def isStructuralNavigationCommand(self, inputEvent=None):
-        """Checks to see if the inputEvent was a structural navigation
-        command. This is necessary to prevent double-presentation of
-        items. [[[TODO - JD: Perhaps this should be moved to default.py]]]
-
-        Arguments:
-        - inputEvent: The input event to examine. If none is provided,
-          orca_state.lastInputEvent will be used.
-
-        Returns True if inputEvent is a structural navigation command
-        enabled in this script.
-        """
-
-        inputEvent = inputEvent or orca_state.lastInputEvent
-        if isinstance(inputEvent, input_event.KeyboardEvent):
-            if self.structuralNavigation.keyBindings.\
-                    getInputHandler(inputEvent):
-                return True
-
-        return False
-
     def doWhereAmI(self, inputEvent, basicOnly):
         """Performs the whereAmI operation."""
 
@@ -319,6 +298,14 @@ class Script(default.Script):
 
         super().doWhereAmI(inputEvent, basicOnly)
 
+    def presentObject(self, obj, **args):
+        if not self._lastCommandWasStructNav:
+            super().presentObject(obj, **args)
+            return
+
+        utterances = self.speechGenerator.generateSpeech(obj, **args)
+        speech.speak(utterances)
+
     def panBrailleLeft(self, inputEvent=None, panAmount=0):
         """In document content, we want to use the panning keys to browse the
         entire document.
@@ -637,6 +624,11 @@ class Script(default.Script):
     def onActiveChanged(self, event):
         """Callback for object:state-changed:active accessibility events."""
 
+        if not event.source.parent:
+            msg = "SOFFICE: Event source lacks parent"
+            debug.println(debug.LEVEL_INFO, msg, True)
+            return
+
         # Prevent this events from activating the find operation.
         # See comment #18 of bug #354463.
         if self.findCommandRun:
@@ -702,6 +694,16 @@ class Script(default.Script):
         if self.utilities.isSameObject(orca_state.locusOfFocus, event.source):
             return
 
+        if self.utilities.isFocusableLabel(event.source):
+            orca.setLocusOfFocus(event, event.source)
+            return
+
+        if self.utilities.isZombie(event.source):
+            comboBox = self.utilities.containingComboBox(event.source)
+            if comboBox:
+                orca.setLocusOfFocus(event, comboBox, True)
+                return
+
         role = event.source.getRole()
 
         # This seems to be something we inherit from Gtk+
@@ -730,7 +732,7 @@ class Script(default.Script):
         if self._inSayAll:
             return
 
-        if self.isStructuralNavigationCommand():
+        if self._lastCommandWasStructNav:
             return
 
         if not event.detail1:
@@ -745,12 +747,19 @@ class Script(default.Script):
             debug.println(debug.LEVEL_INFO, msg, True)
             return
 
+        role = event.source.getRole()
+        if role in [pyatspi.ROLE_TEXT, pyatspi.ROLE_LIST]:
+            comboBox = self.utilities.containingComboBox(event.source)
+            if comboBox:
+                orca.setLocusOfFocus(event, comboBox, True)
+                return
+
         parent = event.source.parent
         if parent and parent.getRole() == pyatspi.ROLE_TOOL_BAR:
             default.Script.onFocusedChanged(self, event)
             return
 
-        role = event.source.getRole()
+        # TODO - JD: Verify this is still needed
         ignoreRoles = [pyatspi.ROLE_FILLER, pyatspi.ROLE_PANEL]
         if role in ignoreRoles:
             return
@@ -775,22 +784,20 @@ class Script(default.Script):
 
         # We should present this in response to active-descendant-changed events
         if event.source.getState().contains(pyatspi.STATE_MANAGES_DESCENDANTS):
-            if role != pyatspi.ROLE_LIST:
-                return
+            return
 
         default.Script.onFocusedChanged(self, event)
 
     def onCaretMoved(self, event):
         """Callback for object:text-caret-moved accessibility events."""
 
-
         if event.detail1 == -1:
             return
 
         if self.utilities._flowsFromOrToSelection(event.source):
            return
 
-        if self.isStructuralNavigationCommand():
+        if self._lastCommandWasStructNav:
             return
 
         if self.utilities.isSpreadSheetCell(orca_state.locusOfFocus):
@@ -854,6 +861,27 @@ class Script(default.Script):
 
         super().onSelectedChanged(event)
 
+    def onSelectionChanged(self, event):
+        """Callback for object:selection-changed accessibility events."""
+
+        if not self.utilities.isComboBoxSelectionChange(event):
+            super().onSelectionChanged(event)
+            return
+
+        selectedChildren = self.utilities.selectedChildren(event.source)
+        if len(selectedChildren) == 1:
+            orca.setLocusOfFocus(event, selectedChildren[0], True)
+
+    def onTextSelectionChanged(self, event):
+        """Callback for object:text-selection-changed accessibility events."""
+
+        if self.utilities.isComboBoxNoise(event):
+            msg = "SOFFICE: Event is believed to be combo box noise"
+            debug.println(debug.LEVEL_INFO, msg, True)
+            return
+
+        super().onTextSelectionChanged(event)
+
     def getTextLineAtCaret(self, obj, offset=None, startOffset=None, endOffset=None):
         """To-be-removed. Returns the string, caretOffset, startOffset."""
 
@@ -895,5 +923,7 @@ class Script(default.Script):
     def onWindowDeactivated(self, event):
         """Callback for window:deactivate accessibility events."""
 
+        self._lastCommandWasStructNav = False
+
         super().onWindowDeactivated(event)
         self.spellcheck.deactivate()
